/* * 
 *  This file is part of Feng
 * 
 * Copyright (C) 2007 by LScube team <team@streaming.polito.it> 
 * See AUTHORS for more details 
 *  
 * Feng is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * Feng is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with Feng; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *  
 * */

#ifndef _RTPH
#define _RTPH

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netembryo/wsocket.h>
#include <fenice/mediathread.h>
#include <bufferpool/bufferpool.h>
#include <fenice/prefs.h>
#define RTP_DEFAULT_PORT 5004
#define RTCP_BUFFERSIZE    1024

typedef enum {
    i_server = 0,
    i_client = 1
} rtcp_index;

typedef struct {
    int RTP;
    int RTCP;
} port_pair;

typedef enum {
    rtp_proto = 0,
    rtcp_proto
} rtp_protos;

typedef struct _RTP_transport {
    Sock *rtp_sock;
    Sock *rtcp_sock;
    struct sockaddr_storage last_stg;
    int rtp_ch, rtcp_ch;
} RTP_transport;

typedef struct _RTCP_stats {
    unsigned int RR_received;
    unsigned int SR_received;
    unsigned long dest_SSRC;
    unsigned int pkt_count;
    unsigned int octet_count;
    int pkt_lost;
    unsigned char fract_lost;
    unsigned int highest_seq_no;
    unsigned int jitter;
    unsigned int last_SR;
    unsigned int delay_since_last_SR;
} RTCP_stats;

typedef struct _RTP_session {
    RTP_transport transport;
    unsigned char rtcp_inbuffer[RTCP_BUFFERSIZE];
    int rtcp_insize;
    unsigned char rtcp_outbuffer[RTCP_BUFFERSIZE];
    uint32_t rtcp_outsize;

    //these time vars now are now back here
    double start_time;
    double seek_time;

    double prev_tx_time;
    unsigned int PreviousCount;
    short MinimumReached;
    short MaximumReached;
    // Back references
    int sched_id;
    unsigned int start_seq;
    unsigned int seq_no;

    unsigned int start_rtptime;

    unsigned char pause;
    unsigned char started;

    unsigned int seq;
    unsigned int ssrc;

    char sd_filename[255]; //!< resource name, including path from avroot

    //mediathread - TODO: find better placement
    Selector *track_selector;

    //multicast management
    int is_multicast; //!< 0, treat as usual, >0 do nothing

    //Consumer has transferred itself here
    BPConsumer *cons;
    RTCP_stats rtcp_stats[2];    //client and server
    struct _RTP_session *next;
} RTP_session;

typedef struct _RTP_header {
    /* byte 0 */
#if (BYTE_ORDER == LITTLE_ENDIAN)
    unsigned char csrc_len:4;   /* expect 0 */
    unsigned char extension:1;  /* expect 1, see RTP_OP below */
    unsigned char padding:1;    /* expect 0 */
    unsigned char version:2;    /* expect 2 */
#elif (BYTE_ORDER == BIG_ENDIAN)
    unsigned char version:2;
    unsigned char padding:1;
    unsigned char extension:1;
    unsigned char csrc_len:4;
#else
#error Neither big nor little
#endif
    /* byte 1 */
#if (BYTE_ORDER == LITTLE_ENDIAN)
    unsigned char payload:7;    /* RTP_PAYLOAD_RTSP */
    unsigned char marker:1;     /* expect 1 */
#elif (BYTE_ORDER == BIG_ENDIAN)
    unsigned char marker:1;
    unsigned char payload:7;
#endif
    /* bytes 2, 3 */
    unsigned short seq_no;
    /* bytes 4-7 */
    unsigned int timestamp;
    /* bytes 8-11 */
    unsigned int ssrc;    /* stream number is used here. */
} RTP_header;

typedef int (*RTP_play_action) (RTP_session * sess);

/**
 * @defgroup RTP
 * @{
 */

/** 
 * RTP ports management functions
 * @defgroup rtp_port RTP ports management functions
 * @{
 */

void RTP_port_pool_init(int port);
int RTP_get_port_pair(port_pair * pair);
int RTP_release_port_pair(port_pair * pair);

/**
 * @}
 */

/** 
 * RTP session management functions
 * @defgroup rtp_session RTP session management functions
 * @{
 */

int RTP_send_packet(RTP_session * session);
RTP_session *RTP_session_destroy(RTP_session *);

//! RTP_transport functions
int RTP_transport_close(RTP_session *);
//! low-level sent/receive packet functions depending on transport settings.
ssize_t RTP_recv(RTP_session *, rtp_protos);

/**
 * @}
 */

/**
 * @}
 */

#endif
